#define minimum(x,y) ( ((x) < (y)) ? (x) : (y) )
#define maximum(x,y) ( ((x) > (y)) ? (x) : (y) )

typedef struct {
	int i;
	double d;
} bn;  /* store big numbers as i*e^d, so d = ln(|bn|) and i is the sign */


void memerror() {
	printf("\n\n** Memory Error **\n\n\n\n");
	exit(1);
}

int factorial(int n) {
	int i, out;
	out = 1;
	for(i=2; i<(n+1); i++) out *= i;
	return out;
}

int random_permutation(int *list, int n) {
	int i, j, k, *tmplist;  /* returns random permutation of a list of integers from 0 to n-1 */
	double tmp;
	if((tmplist = malloc(n*sizeof(int))) == NULL) memerror();
	for(i=0; i<n; i++) tmplist[i] = i;
	for(i=n; i>0; i--) {
		tmp = ((double) i)*drand48();
		j = (int) tmp;
		list[n - i] = tmplist[j];
		for(k=j; k<(i - 1); k++) tmplist[k] = tmplist[k + 1];
	}
	free(tmplist);
	return 0;
}

void intorderlarge(int *input, int *output, int n) { 
	int i, j, tmp;  /* output contains int pointers to input, ordered from largest to smallest */
	for(i=0; i<n; i++) output[i] = i;
	for(i=0; i<(n-1); i++) {
		for(j=(i+1); j<n; j++) {
			if(input[output[i]] < input[output[j]]) {
				tmp = output[j];
				output[j] = output[i];
				output[i] = tmp;
			}
		}
	}
}

void intordersmall(int *input, int *output, int n) { 
	int i, j, tmp;  /* output contains int pointers to input, ordered from smallest to largest */
	for(i=0; i<n; i++) output[i] = i;
	for(i=0; i<(n-1); i++) {
		for(j=(i+1); j<n; j++) {
			if(input[output[i]] > input[output[j]]) {
				tmp = output[j];
				output[j] = output[i];
				output[i] = tmp;
			}
		}
	}
}

void doubleorderlarge(double *input, int *output, int n) { 
	int i, j, tmp;  /* output contains int pointers to input, ordered from largest to smallest */
	for(i=0; i<n; i++) output[i] = i;
	for(i=0; i<(n-1); i++) {
		for(j=(i+1); j<n; j++) {
			if(input[output[i]] < input[output[j]]) {
				tmp = output[j];
				output[j] = output[i];
				output[i] = tmp;
			}
		}
	}
}

void doubleordersmall(double *input, int *output, int n) { 
	int i, j, tmp;  /* output contains double pointers to input, ordered from smallest to largest */
	for(i=0; i<n; i++) output[i] = i;
	for(i=0; i<(n-1); i++) {
		for(j=(i+1); j<n; j++) {
			if(input[output[i]] > input[output[j]]) {
				tmp = output[j];
				output[j] = output[i];
				output[i] = tmp;
			}
		}
	}
}

double lnBinomCoeff(int n, int k) {
	int i;  /* returns ln of n choose k */
	double numer, denom, out;
	numer = 0; denom = 0;
	if (k < 0 || n < 0 || n < k) out = 0;
	else {
		for(i=(k + 1); i<(n + 1); i++) numer += log((double) i);
		for(i=1; i<(n - k + 1); i++) denom += log((double) i);
		out = numer - denom;
	}
	return out;
}

int rowechelon(double **mat, int r, int c) {
	/* returns reduced row echelon form of matrix by Gaussian elimination */
	int i, j, k, l;  
	double tmp, *temprow;
	if ((temprow = malloc(c*sizeof(double))) == NULL) {
		printf("memory error"); exit(0); }
	i=0; j=0;
	while (i < minimum(r,(c-1))) {
		while (mat[i][j] == 0 && j < c) {
			k = i+1;
			while (mat[k][i] == 0 && k < r)
				k++;
			if (k < r) {
				temprow = mat[i];
				mat[i] = mat[k];
				mat[k] = temprow;
			}
			else j++;
		}
		tmp = mat[i][j];
		for(l=j; l<c; l++)
			mat[i][l] /= tmp;
		for(k=0; k<r; k++)
			if(k != i && mat[k][j] != 0) {
				tmp = mat[k][j];
				for(l=j; l<c; l++)
					mat[k][l] -= tmp*mat[i][l];
			}
		i++; if(j < i) j=i;
	}
	free(temprow);
	return 0;
}

int LUinverse(double **a, double **b, int n) {  
	/* Inverts the matrix a into the matrix b */
	int i, j, k, q, row, col, *rows, *cols, zero;
	double **UL, *temprow, m, tmp;
	if ((rows = malloc(n*sizeof(int))) == NULL) memerror();
	if ((cols = malloc(n*sizeof(int))) == NULL) memerror();
	if ((temprow = malloc(n*sizeof(double))) == NULL) memerror();
	if ((UL = malloc(n*sizeof(double *))) == NULL) memerror();
	for (i=0; i<n; i++) if ((UL[i] = malloc(n*sizeof(double))) == NULL) memerror();
	for(i=0; i<n; i++) {
		rows[i] = i; cols[i] = i;
		for(j=0; j<n; j++) b[i][j] = a[i][j];
	}
	zero = 0;
	for(i=0; i<n; i++) {
		m=0; row=i; col=i;
		for(k=i; k<n; k++) {  /* Find largest element */
			for(j=i; j<n; j++) {
				tmp = fabs(b[k][j]);
				if(tmp > m) {m = tmp; row = k; col = j; }
			}
		}
		if(m == 0) {zero = 1; continue; }
		if(row != i) {  /*switch rows */
			for(k=0; k<n; k++) {temprow[k] = b[row][k]; b[row][k] = b[i][k]; b[i][k] = temprow[k]; }
			q = rows[i]; rows[i] = rows[row]; rows[row] = q;
		}
		if(col != i) {  /*switch columns */
			for(j=0; j<n; j++) {
				m = b[j][col]; b[j][col] = b[j][i]; b[j][i] = m;
			}
			q = cols[i]; cols[i] = cols[col]; cols[col] = q;
		}
		for(j=i+1; j<n; j++)  /* LU decomposition */
			b[j][i] = b[j][i]/b[i][i];
		for(j=i+1; j<n; j++) {
			for(k=i+1; k<n; k++)
				b[j][k] = b[j][k] - b[j][i]*b[i][k];
		}
	}
	if(zero == 1) return zero;
	for(i=1; i<n; i++) { /* invert lower factor */
		for(j=0; j<i; j++) {
			tmp = 0; for(k=(j+1); k<i; k++) tmp += b[i][k]*UL[k][j];
			UL[i][j] = - tmp - b[i][j];
		}
	}
	for(i=(n-1); i>=0; i--) { /* invert upper factor */
		UL[i][i] = 1/b[i][i];
		for(j=(i+1); j<n; j++) {
			tmp = 0; for(k=(i+1); k<(j+1); k++) tmp += b[i][k]*UL[k][j];
			UL[i][j] = -1 * UL[i][i] * tmp;
		}
	}
	for(i=0; i<n; i++) { /*multiply inverses of U and L */
		for(j=0; j<n; j++) {
			tmp = 0; for(k=maximum((j+1),i); k<n; k++) tmp += UL[i][k]*UL[k][j];
			if(i<=j) b[i][j] = UL[i][j] + tmp;
			else b[i][j] = tmp;
		}
	}
	for(i=0; i<n; i++) for(j=0; j<n; j++) UL[cols[i]][j] = b[i][j];
	for(j=0; j<n; j++) for(i=0; i<n; i++) b[i][rows[j]] = UL[i][j];
	for (i=0; i<n; i++) free(UL[i]);
	free(rows); free (cols); free(temprow); free (UL);
	return zero;
}

int eigen(double **mat, double **out, int n) {
	/* returns eigenvectors and eigenvalues of mat as columns of out -- symmetric only!!! */
	int i, j, k, l;   
	double c, s, tmp, tmp1, rot, **tmpmat;
	if ((tmpmat = malloc((n+1)*sizeof(double *))) == NULL) {
		printf("memory error"); exit(0); }
	for (i=0; i<(n+1); i++)
		if ((tmpmat[i] = malloc((n+1)*sizeof(double))) == NULL) {
			printf ("memory error"); exit(0); }
	for(i=0; i<n; i++)
		for(j=(i+1); j<n; j++)
			if(mat[i][j] != mat[j][i]) return 1;  /* return 1 if not symmetric */
	if (n == 1) {out[0][0] = 1; out[0][1] = mat[0][0]; }
	else if (n == 2) {
		tmp = sqrt(4*mat[0][1]*mat[0][1] + (mat[0][0] - mat[1][1])*(mat[0][0] - mat[1][1]));
		out[2][0] = 0.5*(mat[0][0] + mat[1][1] + tmp);
		out[2][1] = 0.5*(mat[0][0] + mat[1][1] - tmp);
		for(i=0; i<2; i++) {
			if(mat[0][1] != 0) {
				tmp = (out[2][i] - mat[0][0])/mat[0][1];
				tmp1 = sqrt(1 + tmp*tmp);
				out[0][i] = 1/tmp1; out[1][i] = tmp/tmp1;
			}
			else { out[i][i] = 1; out[1-i][1-i] = 0; }
		}
	}
	else {  /* Jacobi algorithm for finding eigenvalues */
		for(i=0; i<n; i++)
			for(j=0; j<n; j++)
				tmpmat[i][j] = mat[i][j];
		k = 0; l = 1; tmp = tmpmat[k][l];
		for (i=0; i<n; i++) {
			for (j=(i+1); j<n; j++) {
				if (fabs(tmpmat[i][j]) > fabs(tmp)) {
					k = i; l = j; tmp = tmpmat[i][j]; }
			}
		}
		while (fabs(tmp) > 0.00001*sqrt(fabs(tmpmat[k][k]*tmpmat[l][l]))) {
			if (tmpmat[k][k] == tmpmat[l][l]) rot = 0.25*M_PI;
			else rot = 0.5*atan(2*tmpmat[k][l]/(tmpmat[k][k]-tmpmat[l][l]));
			c = cos(rot); s = sin(rot);  
			/* lower triangle is rotated, then transfered to upper; k<l */
			for (i=0; i<k; i++) {
				tmpmat[k][i] = c*tmpmat[i][k] + s*tmpmat[i][l];
				tmpmat[l][i] = -1*s*tmpmat[i][k] + c*tmpmat[i][l];
			}
			for (i=(k+1); i<l; i++) {
				tmpmat[i][k] = c*tmpmat[k][i] + s*tmpmat[i][l];
				tmpmat[l][i] = -1*s*tmpmat[k][i] + c*tmpmat[i][l];
			}
			for (i=(l+1); i<n; i++) {
				tmpmat[i][k] = c*tmpmat[k][i] + s*tmpmat[l][i];
				tmpmat[i][l] = -1*s*tmpmat[k][i] + c*tmpmat[l][i];
			}
			tmp = tmpmat[k][k]; tmp1 = tmpmat[l][l];
			tmpmat[l][k] = (c*c - s*s)*tmpmat[k][l] + c*s*(tmp1 - tmp);
			tmpmat[k][k] = c*c*tmp + 2*c*s*tmpmat[k][l] + s*s*tmp1;
			tmpmat[l][l] = s*s*tmp - 2*s*c*tmpmat[k][l] + c*c*tmp1;
			for (i=0; i<(n-1); i++)
				for (j=(i+1); j<n; j++)
					tmpmat[i][j] = tmpmat[j][i];
			k = 0; l = 1; tmp = tmpmat[k][l];
			for (i=0; i<n; i++) {
				for (j=(i+1); j<n; j++) {
					if (fabs(tmpmat[i][j]) > fabs(tmp)) {
						k = i; l = j; tmp = tmpmat[i][j]; }
				}
			}
		}
		for(i=0; i<n; i++) {  /* order eigenvalues by absolute value */
			k=0; tmp = fabs(tmpmat[k][k]);
			for(j=1; j<n; j++) {
				if(fabs(tmpmat[j][j]) > tmp) {
					tmp = fabs(tmpmat[j][j]); k = j;
				}
			}	
			out[n][i] = tmpmat[k][k];
			tmpmat[k][k] = 0;
		}
		for(i=0; i<n; i++)  {  /* Calculate eigenvectors */
			for(j=0; j<n; j++) {
				for(k=0; k<n; k++)
					tmpmat[j+1][k] = mat[j][k];
				tmpmat[j+1][j] -= out[n][i];
				tmpmat[0][j] = 1;
				tmpmat[j+1][n] = 0;
			}
			tmpmat[0][n] = 1;
			j = rowechelon(tmpmat, (n+1), (n+1));
			tmp = 0;
			for(j=0; j<n; j++) tmp += tmpmat[j][n]*tmpmat[j][n];
			for(j=0; j<n; j++)
				out[j][i] = tmpmat[j][n]/sqrt(tmp);
		}
	}
	for(i=0; i<(n+1); i++) free(tmpmat[i]); free(tmpmat);
	return 0;
}

void MatrixxVector(double **matrix, double *vector, double *out, int r, int c) {
	int i, j; double tmp;
	for(i=0; i<r; i++) {
		tmp = 0;
		for(j=0; j<c; j++) tmp += matrix[i][j]*vector[j];
		out[i] = tmp;
	}
}

double CenterandRotate(double **points, double **outpoints, int n, int d) {
	/* centers and rotates n vectors in d dimensions given in points[n][d], populates new vectors into outpoints[n][d], returns Kirkpatrick's n_D */
	int i, j, k;  
	double tmp, **matrix, **evecs, *mean, out;
	if((matrix = malloc(d*sizeof(double *))) == NULL) memerror();
	for(i=0; i<d; i++) if((matrix[i] = malloc(d*sizeof(double))) == NULL) memerror();
	if((evecs = malloc((d+1)*sizeof(double *))) == NULL) memerror();
	for(i=0; i<d+1; i++) if((evecs[i] = malloc(d*sizeof(double))) == NULL) memerror();
	if((mean = malloc(d*sizeof(double))) == NULL) memerror();
	for(i=0; i<d; i++) {
		tmp = 0; for(j=0; j<n; j++) tmp += points[j][i];
		mean[i] = tmp/((double) n);
	}
	for(i=0; i<n; i++) for(j=0; j<d; j++) outpoints[i][j] = points[i][j] - mean[j];
	for(i=0; i<d; i++)
		for(j=i; j<d; j++) {
			tmp = 0; for(k=0; k<n; k++) tmp += outpoints[k][i]*outpoints[k][j];
			matrix[i][j] = tmp/((double) n);
			matrix[j][i] = matrix[i][j];
		}
	k = 0;
	k = eigen(matrix, evecs, d);
	tmp = 0; for(i=0; i<d; i++) tmp += evecs[d][i];
	out = tmp/evecs[d][0];
	k = LUinverse(evecs, matrix, d);
	for(i=0; i<n; i++) {
		MatrixxVector(matrix, outpoints[i], mean, d, d);
		for(j=0; j<d; j++) outpoints[i][j] = mean[j];
	}
	for(i=0; i<d; i++) free(matrix[i]);
	for(i=0; i<(d+1); i++) free(evecs[i]);
	free(matrix); free(evecs); free(mean);
	return out;
}

double sq_dist(double *point1, double *point2, int dim) {
	int i;  /* squared Euclidean distance btw two points */
	double tmp;
	tmp = 0;
	for(i=0; i<dim; i++) tmp += (point1[i] - point2[i])*(point1[i] - point2[i]);
	return tmp;
}

double ln_factorial(int n) {
	int i;
	double out;
	out = 0;
	for(i=2; i<(n+1); i++) out += log((double) i);
	return out;
}

double Poisson_constant(int **data, int spp){
	/* return sum of ln factorials of k_ij */
	int i,j;
	double out;
	out = 0;
	for(i=0; i<spp; i++) for(j=0; j<spp; j++) out += ln_factorial(data[i][j]);
	return out;
}

double Poisson_max_p(int **data, double **points, int spp, int dim) {
	int i, j;  /* returns MLE for p in Poisson model */
	double denom, numer;
	numer = 0; denom = 0;
	for(i=0; i<spp; i++) for(j=0; j<spp; j++) numer += data[i][j];
	for(i=0; i<spp; i++) for(j=0; j<spp; j++) denom += 1/exp(sq_dist(points[i], points[spp + j], dim));
	return numer/denom;
}

double BinomlnL_single(double distance, int *data, double c) {
		/* returns lnL for single squared distance btw two points, not including binomial coefficient */
	return (data[0]*log(c) - data[0]*distance + data[1]*log(1 - c*exp(-1*distance)));
}

double PoissonlnL_single(double distance, int data, double p) {
		/* returns lnL for single squared distance btw two points, not including factorial constant */
	return (data*log(p) - data*distance - p*exp(-distance));
}

double BinomlnL_point(int ***data, double **points, int which, int spp, int dim, double c) {
	int i;  /* returns contribution of point[which] to total lnL */
	double tmp;
	tmp = 0;
	if(which < spp)  /* point[which] is female */
		for(i=0; i<spp; i++) 
			tmp += BinomlnL_single(sq_dist(points[which],points[spp + i],dim), data[which][i], c);
	else
		for(i=0; i<spp; i++)
			tmp += BinomlnL_single(sq_dist(points[i],points[which],dim), data[i][which - spp], c);
	return tmp;
}

double PoissonlnL_point(int **data, double **points, int which, int spp, int dim, double p) {
	int i;  /* returns contribution of point[which] to total lnL */
	double tmp;
	tmp = 0;
	if(which < spp)  /* point[which] is female */
		for(i=0; i<spp; i++) 
			tmp += PoissonlnL_single(sq_dist(points[which],points[spp + i],dim), data[which][i], p);
	else
		for(i=0; i<spp; i++)
			tmp += PoissonlnL_single(sq_dist(points[i],points[which],dim), data[i][which - spp], p);
	return tmp;

}

double BinomlnL_total(int ***data, double **points, int spp, int dim, double c) {
	int i, j;  /* returns total lnL of points */
	double out;
	out = 0;
	for(i=0; i<spp; i++) 
		for(j=0; j<spp; j++) 
			out += BinomlnL_single(sq_dist(points[i], points[spp + j], dim), data[i][j], c);
	return out;
}

double PoissonlnL_total(int **data, double **points, int spp, int dim, double p) {
	int i, j;  /* returns total lnL of points */
	double out;
	out = 0;
	for(i=0; i<spp; i++) 
		for(j=0; j<spp; j++) 
			out += PoissonlnL_single(sq_dist(points[i], points[spp + j], dim), data[i][j], p);
	return out;
}

double Binom_maxlnL_point(int *data) {
	double k, l, n;  /* returns lnL for MLE of distance btw points = sqrt(-ln(k/cn)) */
	k = (double) data[0];
	l = (double) data[1];
	n = (double) data[2];
	return k*log(k/n) + l*log(l/n);
}

double Poisson_maxlnL_point(int data) {
	double k;  /* returns lnL for MLE of distance btw points = sqrt(ln p - ln k), not including factorial constant */
	k = (double) data;
	return k*(log(k) - 1);
}

double Binom_cost_point(int ***data, double **points, int which, int spp, int dim, double c) {
	int i;
	double maxlnL;
	maxlnL = 0;
	if(which < spp) for(i=0; i<spp; i++) maxlnL += Binom_maxlnL_point(data[which][i]);
	else for(i=0; i<spp; i++) maxlnL += Binom_maxlnL_point(data[i][which - spp]);
	return (maxlnL - BinomlnL_point(data, points, which, spp, dim, c));
}

double Poisson_cost_point(int **data, double **points, int which, int spp, int dim, double p) {
	int i;
	double maxlnL;
	maxlnL = 0;
	if(which < spp) for(i=0; i<spp; i++) maxlnL += Poisson_maxlnL_point(data[which][i]);
	else for(i=0; i<spp; i++) maxlnL += Poisson_maxlnL_point(data[i][which - spp]);
	return (maxlnL - PoissonlnL_point(data, points, which, spp, dim, p));
}

int Binom_max_cost(int ***data, double **points, int spp, int dim, double c) {
	int i,j;  /* returns ID of point with largest lnL cost (difference from max lnL) */
	double tmp1, tmp2;
	j = 0;
	tmp1 = Binom_cost_point(data,points,j,spp,dim,c);
	for(i=1; i<2*spp; i++) if((tmp2 = Binom_cost_point(data,points,i,spp,dim,c)) > tmp1) {tmp1 = tmp2; j = i; }
	return j;
}

int Poisson_max_cost(int **data, double **points, int spp, int dim, double p) {
	int i,j;  /* returns ID of point with largest lnL cost (difference from max lnL) */
	double tmp1, tmp2;
	j = 0;
	tmp1 = Poisson_cost_point(data,points,j,spp,dim,p);
	for(i=1; i<2*spp; i++) if((tmp2 = Poisson_cost_point(data,points,i,spp,dim,p)) > tmp1) {tmp1 = tmp2; j = i; }
	return j;
}

int BinomlnL_point_gradient(int ***data, double **points, int which, int spp, int dim, double c, double dx, double *grad) {
	if(dx == 0) return 1;  /* returns gradient of lnL surface for point[which], populates grad */
	int i,j,k;
	double like0, like1, **tmppoints;
	if((tmppoints = malloc(2*spp*sizeof(double *))) == NULL) memerror();
	for(i=0; i<2*spp; i++) if((tmppoints[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) tmppoints[j][k] = points[j][k];
	for(i=0; i<dim; i++) {
		tmppoints[which][i] += dx;
		like1 = BinomlnL_point(data, tmppoints, which, spp, dim, c);
		tmppoints[which][i] -= dx;
		like0 = BinomlnL_point(data, tmppoints, which, spp, dim, c);
		grad[i] = (like1 - like0)/dx;
	}
	for(j=0; j<2*spp; j++) free(tmppoints[j]); free(tmppoints);
	return 0;
}

int PoissonlnL_point_gradient(int **data, double **points, int which, int spp, int dim, double p, double dx, double *grad) {
	if(dx == 0) return 1;  /* returns gradient of lnL surface for point[which], populates grad */
	int i,j,k;
	double like0, like1, **tmppoints;
	if((tmppoints = malloc(2*spp*sizeof(double *))) == NULL) memerror();
	for(i=0; i<2*spp; i++) if((tmppoints[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) tmppoints[j][k] = points[j][k];
	for(i=0; i<dim; i++) {
		tmppoints[which][i] += dx;
		like1 = PoissonlnL_point(data, tmppoints, which, spp, dim, p);
		tmppoints[which][i] -= dx;
		like0 = PoissonlnL_point(data, tmppoints, which, spp, dim, p);
		grad[i] = (like1 - like0)/dx;
	}
	for(j=0; j<2*spp; j++) free(tmppoints[j]); free(tmppoints);
	return 0;
}

int BinomlnL_norm_gradient(int ***data, double **points, int which, int spp, int dim, double c, double dx, double *grad) {
	if(dx == 0) return 1;  /* returns normalized (to length 1) gradient of lnL surface for point [which], populates grad */
	int i;
	double norm;
	i = BinomlnL_point_gradient(data, points, which, spp, dim, c, dx, grad);
	norm = 0; for(i=0; i<dim; i++) norm += grad[i]*grad[i]; norm = sqrt(norm);
	if(norm > 0) for(i=0; i<dim; i++) grad[i] /= norm;
	return 0;
}

int PoissonlnL_norm_gradient(int **data, double **points, int which, int spp, int dim, double p, double dx, double *grad) {
	if(dx == 0) return 1;  /* returns normalized (to length 1) gradient of lnL surface for point [which], populates grad */
	int i;
	double norm;
	i = PoissonlnL_point_gradient(data, points, which, spp, dim, p, dx, grad);
	norm = 0; for(i=0; i<dim; i++) norm += grad[i]*grad[i]; norm = sqrt(norm);
	if(norm > 0) for(i=0; i<dim; i++) grad[i] /= norm;
	return 0;
}

int BinomlnL_Hessian(int ***data, double **points, int which, int spp, int dim, double c, double dx, double **matrix) {
	if(dx == 0) return 1;  /* returns Hessian matrix for lnL surface for point[which], populates matrix */
	int i, j, k;
	double dx2, likehere, liketmp, *likedx, **tmppoints;
	if((likedx = malloc(dim*sizeof(double))) == NULL) memerror();
	if((tmppoints = malloc(2*spp*sizeof(double *))) == NULL) memerror();
	for(i=0; i<2*spp; i++) if((tmppoints[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) tmppoints[j][k] = points[j][k];
	likehere = BinomlnL_point(data, tmppoints, which, spp, dim, c);
	dx2 = dx*dx;
	for(i=0; i<dim; i++) {
		tmppoints[which][i] += dx;
		likedx[i] = BinomlnL_point(data, tmppoints, which, spp, dim, c);
		tmppoints[which][i] = points[which][i] - dx;
		liketmp = BinomlnL_point(data, tmppoints, which, spp, dim, c);
		tmppoints[which][i] = points[which][i];
		matrix[i][i] = (likedx[i] + liketmp - 2*likehere)/dx2;
	}
	for(i=0; i<dim; i++) {
		for(j=(i+1); j<dim; j++) {
			tmppoints[which][i] += dx; tmppoints[which][j] += dx;
			liketmp = BinomlnL_point(data, tmppoints, which, spp, dim, c);
			tmppoints[which][i] = points[which][i]; tmppoints[which][j] = points[which][j];
			matrix[i][j] = (liketmp - likedx[i] - likedx[j] + likehere)/dx2;
			matrix[j][i] = matrix[i][j];
		}
	}
	for(j=0; j<2*spp; j++) free(tmppoints[j]); free(tmppoints); free(likedx);
	return 0;
}

int PoissonlnL_Hessian(int **data, double **points, int which, int spp, int dim, double p, double dx, double **matrix) {
	if(dx == 0) return 1;  /* returns Hessian matrix for lnL surface for point[which], populates matrix */
	int i, j, k;
	double dx2, likehere, liketmp, *likedx, **tmppoints;
	if((likedx = malloc(dim*sizeof(double))) == NULL) memerror();
	if((tmppoints = malloc(2*spp*sizeof(double *))) == NULL) memerror();
	for(i=0; i<2*spp; i++) if((tmppoints[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) tmppoints[j][k] = points[j][k];
	likehere = PoissonlnL_point(data, tmppoints, which, spp, dim, p);
	dx2 = dx*dx;
	for(i=0; i<dim; i++) {
		tmppoints[which][i] += dx;
		likedx[i] = PoissonlnL_point(data, tmppoints, which, spp, dim, p);
		tmppoints[which][i] = points[which][i] - dx;
		liketmp = PoissonlnL_point(data, tmppoints, which, spp, dim, p);
		tmppoints[which][i] = points[which][i];
		matrix[i][i] = (likedx[i] + liketmp - 2*likehere)/dx2;
	}
	for(i=0; i<dim; i++) {
		for(j=(i+1); j<dim; j++) {
			tmppoints[which][i] += dx; tmppoints[which][j] += dx;
			liketmp = PoissonlnL_point(data, tmppoints, which, spp, dim, p);
			tmppoints[which][i] = points[which][i]; tmppoints[which][j] = points[which][j];
			matrix[i][j] = (liketmp - likedx[i] - likedx[j] + likehere)/dx2;
			matrix[j][i] = matrix[i][j];
		}
	}
	for(j=0; j<2*spp; j++) free(tmppoints[j]); free(tmppoints); free(likedx);
	return 0;
}

double Binom_point_Newton_approx(int ***data, double **points, int which, int spp, int dim, double c, double dx, double **newpoints) {
	if(dx == 0) return 1;
	int i, j, k;  /* conducts Newton-Raphson on lnL for point[which], populates newpoints with projection, return lnL for point[which] */
	double *gradient, **Hessian, **invHessian, *vector;
	if((gradient = malloc(dim*sizeof(double))) == NULL) memerror();
	if((vector = malloc(dim*sizeof(double))) == NULL) memerror();
	if((Hessian = malloc(dim*sizeof(double *))) == NULL) memerror();
	for(i=0; i<dim; i++) if((Hessian[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	if((invHessian = malloc(dim*sizeof(double *))) == NULL) memerror();
	for(i=0; i<dim; i++) if((invHessian[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	i = BinomlnL_point_gradient(data, points, which, spp, dim, c, dx, gradient);
	i = BinomlnL_Hessian(data, points, which, spp, dim, c, dx, Hessian);
	i = LUinverse(Hessian, invHessian, dim);
	if(i == 1) return 0.0;
	MatrixxVector(invHessian, gradient, vector, dim, dim);
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) newpoints[j][k] = points[j][k];
	for(k=0; k<dim; k++) newpoints[which][k] -= vector[k];
	for(i=0; i<dim; i++) {free(Hessian[i]); free(invHessian[i]); }
	free(gradient); free(Hessian); free(invHessian); free(vector);
	return BinomlnL_point(data, newpoints, which, spp, dim, c);
}

double Poisson_point_Newton_approx(int **data, double **points, int which, int spp, int dim, double p, double dx, double **newpoints) {
	if(dx == 0) return 1;
	int i, j, k;  /* conducts Newton-Raphson on lnL for point[which], populates newpoints with projection, return lnL for point[which] */
	double *gradient, **Hessian, **invHessian, *vector;
	if((gradient = malloc(dim*sizeof(double))) == NULL) memerror();
	if((vector = malloc(dim*sizeof(double))) == NULL) memerror();
	if((Hessian = malloc(dim*sizeof(double *))) == NULL) memerror();
	for(i=0; i<dim; i++) if((Hessian[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	if((invHessian = malloc(dim*sizeof(double *))) == NULL) memerror();
	for(i=0; i<dim; i++) if((invHessian[i] = malloc(dim*sizeof(double))) == NULL) memerror();
	i = PoissonlnL_point_gradient(data, points, which, spp, dim, p, dx, gradient);
	i = PoissonlnL_Hessian(data, points, which, spp, dim, p, dx, Hessian);
	i = LUinverse(Hessian, invHessian, dim);
	if(i == 1) return 0.0;
	MatrixxVector(invHessian, gradient, vector, dim, dim);
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) newpoints[j][k] = points[j][k];
	for(k=0; k<dim; k++) newpoints[which][k] -= vector[k];
	for(i=0; i<dim; i++) {free(Hessian[i]); free(invHessian[i]); }
	free(gradient); free(Hessian); free(invHessian); free(vector);
	return PoissonlnL_point(data, newpoints, which, spp, dim, p);
}

double Binom_point_gradient_approx(int ***data, double **points, int which, int spp, int dim, double c, double dx, double gamma, double **newpoints) {
	if(dx == 0) return 1;  /* uses gradient method to step forward by gamma, populates newpoints, return lnL for new point[which] */
	int j, k;
	double *gradient;
	if((gradient = malloc(dim*sizeof(double))) == NULL) memerror();
	j = BinomlnL_norm_gradient(data, points, which, spp, dim, c, dx, gradient);
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) newpoints[j][k] = points[j][k];
	for(k=0; k<dim; k++) newpoints[which][k] += gamma*gradient[k];
	free(gradient);
	return BinomlnL_point(data, newpoints, which, spp, dim, c);
}

double Poisson_point_gradient_approx(int **data, double **points, int which, int spp, int dim, double p, double dx, double gamma, double **newpoints) {
	if(dx == 0) return 1;  /* uses gradient method to step forward by gamma, populates newpoints, return lnL for new point[which] */
	int j, k;
	double *gradient;
	if((gradient = malloc(dim*sizeof(double))) == NULL) memerror();
	j = PoissonlnL_norm_gradient(data, points, which, spp, dim, p, dx, gradient);
	for(j=0; j<2*spp; j++) for(k=0; k<dim; k++) newpoints[j][k] = points[j][k];
	for(k=0; k<dim; k++) newpoints[which][k] += gamma*gradient[k];
	free(gradient);
	return PoissonlnL_point(data, newpoints, which, spp, dim, p);
}

